package com.telpo.dipperposition.handler;

import com.telpo.dipperposition.common.HexConvert;
import com.telpo.dipperposition.config.PositionConfig;
import com.telpo.dipperposition.service.IDipperAstPosAsyncTaskService;
import com.telpo.dipperposition.service.IDipperAstTimeAsyncTaskService;
import com.telpo.dipperposition.service.IDipperDataAsyncTaskService;
import io.netty.buffer.ByteBuf;
import io.netty.buffer.ByteBufAllocator;
import io.netty.buffer.CompositeByteBuf;
import io.netty.buffer.Unpooled;
import io.netty.channel.socket.SocketChannel;
import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.ChannelInboundHandlerAdapter;
import io.netty.util.CharsetUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;

/**
 * @program: dipperposition
 * @description: Netty服务器处理句柄
 * @author: linwl
 * @create: 2021-01-13 13:56
 **/
@Slf4j
@Component
public class NettyServerHandler extends ChannelInboundHandlerAdapter {

    private static String AST_TIME_CMD = "TIME";
    private static String AST_POS_CMD = "POS";
    private static String AST_EPH_CMD = "EPH";
    private static String AST_ALL_CMD = "ALL";
    private static String AST_TIME_CMD_BYTE = "54494d45";
    private static String AST_POS_CMD_BYTE = "504f53";
    private static String AST_EPH_CMD_BYTE = "455048";
    private static String AST_ALL_CMD_BYTE = "616C6C";

    @Autowired
    private PositionConfig positionConfig;
    @Autowired
    private IDipperAstTimeAsyncTaskService dipperTimeAsyncTaskService;
    @Autowired
    private IDipperAstPosAsyncTaskService dipperAstPosAsyncTaskService;
    @Autowired
    private IDipperDataAsyncTaskService dipperDataAsyncTaskService;

    private static NettyServerHandler nettyServerHandler;
    public NettyServerHandler() {}
    @PostConstruct
    public void init() {
        nettyServerHandler = this;
        nettyServerHandler.positionConfig = this.positionConfig;
        nettyServerHandler.dipperTimeAsyncTaskService = this.dipperTimeAsyncTaskService;
        nettyServerHandler.dipperAstPosAsyncTaskService = this.dipperAstPosAsyncTaskService;
        nettyServerHandler.dipperDataAsyncTaskService = this.dipperDataAsyncTaskService;
    }

    /**
     * 客户端连接会触发
     */
    @Override
    public void channelActive(ChannelHandlerContext ctx) throws Exception {
        log.info("Channel active......");

        SocketChannel channel = (SocketChannel) ctx.channel();
        log.info("链接报告开始");
        log.info("链接报告信息：有一客户端链接到本服务端");
        log.info("链接报告IP:" + channel.localAddress().getHostString());
        log.info("链接报告Port:" + channel.localAddress().getPort());
        log.info("链接报告完毕");
        //通知客户端链接建立成功
        // 默认返回取得时间成功
        // String ackAckCheckRef = "233E0101020004020A1D";
        //String ackAckCheckRef = "23 3E 01 01 02 00 04 21 29 3C";
        //ByteBuf buf = Unpooled.buffer(ackAckCheckRef.getBytes().length);
        //buf.writeBytes(ackAckCheckRef.getBytes(CharsetUtil.UTF_8));
        //ctx.writeAndFlush(buf);
    }


    /**
     * 当客户端主动断开服务端的链接后，这个通道就是不活跃的。
     * 也就是说客户端与服务端的关闭了通信通道并且不可以传输数据
     */
    @Override
    public void channelInactive(ChannelHandlerContext ctx) throws Exception {
        log.info("客户端断开链接，IP：{}", ctx.channel().localAddress().toString());
        ctx.channel().close();
    }

    /**
     * 客户端发消息会触发
     */
    @Override
    public void channelRead(ChannelHandlerContext ctx, Object msg) throws Exception {
        //接收msg消息与上一章节相比，此处已经不需要自己进行解码}
        SocketChannel channel = (SocketChannel) ctx.channel();
        ByteBufAllocator bba = channel.config().getAllocator();
        String ipAddress = channel.remoteAddress().getHostString();
        log.info("接收到消息:" + msg + ",客户端IP:" + ipAddress);
        if(ipAddress.contains(":")) {
            String[] ipAddressArr = ipAddress.split(":");
            ipAddress = ipAddressArr[0].replace("/","");
        }
        log.info("新客户端IP:" + ipAddress);

        String channelAns = "";
        // 返回时间指令
        ByteBuf buf = null;
        if (AST_TIME_CMD.equals(msg)) {
            // 初始时间辅助输入;
            channelAns = nettyServerHandler.dipperTimeAsyncTaskService.pushAstTime();
            //log.debug(channelAns);
            if (channelAns != null) {
//                buf = bba.buffer(channelAns.getBytes().length);
                byte[] returnBytes = getReturnBytes(channelAns);
                buf = bba.buffer(returnBytes.length + 4);
                //Unpooled.buffer(channelAns.getBytes().length);
//                buf.writeBytes(channelAns.getBytes(CharsetUtil.UTF_8));
                buf.writeBytes(returnBytes);
                buf.writeBytes(getCheckSumBytes(channelAns));
                buf.writeBytes(getReturnBytes());
                // 确保通道处于活跃状态
                // 不活跃状态继续写会产生CLOSE_WAIT现象
                if(ctx.channel().isActive()) {
                    ctx.write(buf);
                    ctx.flush();
                }
            }
        }
        // 发送SDBP-AST-POS获取辅助位置信息
        if (AST_POS_CMD.equals(msg)) {
            channelAns = nettyServerHandler.dipperAstPosAsyncTaskService.pushAstPos(ipAddress);
            //log.debug(channelAns);
            if (channelAns != null) {
//                buf = bba.buffer(channelAns.getBytes().length);
                byte[] returnBytes = getReturnBytes(channelAns);
                buf = bba.buffer(returnBytes.length + 4);
                //Unpooled.buffer(channelAns.getBytes().length);
//                buf.writeBytes(channelAns.getBytes(CharsetUtil.UTF_8));
                buf.writeBytes(returnBytes);
                buf.writeBytes(getCheckSumBytes(channelAns));
                buf.writeBytes(getReturnBytes());
                // 确保通道处于活跃状态
                // 不活跃状态继续写会产生CLOSE_WAIT现象
                if(ctx.channel().isActive()) {
                    ctx.write(buf);
                    ctx.flush();
                }
            }
        }

        // 从缓存获取SDBP-AST-EPH星历数
        if (AST_EPH_CMD.equals(msg)) {
            //channelAns = nettyServerHandler.dipperDataAsyncTaskService.getAstEPH();
            byte[] returnBytes = nettyServerHandler.dipperDataAsyncTaskService.getAstEPH();
            //log.debug(channelAns);
            //if (channelAns != null) {
            //    buf = bba.buffer(channelAns.getBytes().length);
            buf = bba.buffer(returnBytes.length);
                //Unpooled.buffer(channelAns.getBytes().length);
            buf.writeBytes(returnBytes);
            // 确保通道处于活跃状态
            // 不活跃状态继续写会产生CLOSE_WAIT现象
            if(ctx.channel().isActive()) {
                ctx.write(buf);
                ctx.flush();
            }
            //}
        }

        // 最后把SDBP-AST-TIME、SDBP-AST-POS、SDBP-AST-EPH并包一起发给设备。
        // 设备采用16进制获取数据，则代理服务器也是采用16进制返回数据。
        // 通知客户端链消息发送成功
        if (AST_ALL_CMD.equals(msg.toString().toUpperCase())) {
            channelAns = nettyServerHandler.dipperTimeAsyncTaskService.pushAstTime();

            if (channelAns != null) {
                byte[] returnBytes = getReturnBytes(channelAns);
                buf = bba.buffer(returnBytes.length + 4);
                buf.writeBytes(returnBytes);
                buf.writeBytes(getCheckSumBytes(channelAns));
                buf.writeBytes(getReturnBytes());

                // 确保通道处于活跃状态
                // 不活跃状态继续写会产生CLOSE_WAIT现象
                if(ctx.channel().isActive()) {
                    ctx.write(buf);
                    ctx.flush();
                }
            }
            channelAns = nettyServerHandler.dipperAstPosAsyncTaskService.pushAstPos(ipAddress);

            if (channelAns != null) {
                byte[] returnBytes = getReturnBytes(channelAns);
                buf = bba.buffer(returnBytes.length + 4);
                buf.writeBytes(returnBytes);
                buf.writeBytes(getCheckSumBytes(channelAns));
                buf.writeBytes(getReturnBytes());
                // 确保通道处于活跃状态
                // 不活跃状态继续写会产生CLOSE_WAIT现象
                if(ctx.channel().isActive()) {
                    ctx.write(buf);
                }
            }

            byte[] returnBytes = nettyServerHandler.dipperDataAsyncTaskService.getAstEPH();
            //if (channelAns != null) {
            log.debug("PEH Buffer Length is:" + returnBytes.length);
            // 使用池化的堆内存，以减少内存碎片
            //ByteBuf channelPehAnsBuf = Unpooled.buffer(channelAns.getBytes().length);
            //ByteBuf channelPehAnsBuf = bba.buffer(channelAns.getBytes().length);
            ByteBuf channelPehAnsBuf = bba.buffer(returnBytes.length);
            //channelPehAnsBuf.writeBytes(channelAns.getBytes(CharsetUtil.UTF_8));
            channelPehAnsBuf.writeBytes(returnBytes);
            // compositeByteBuf.addComponent(channelPehAnsBuf);
            // log.info("CompositeByteBuf Length is:" + compositeByteBuf.capacity());
            // 确保通道处于活跃状态
            // 不活跃状态继续写会产生CLOSE_WAIT现象
            if(ctx.channel().isActive()) {
                ctx.write(channelPehAnsBuf);
                ctx.flush();
            }
            //}

            // 写给下一个Handler，最后一个Handler将内容移出pipeline
            // ctx.writeAndFlush(compositeByteBuf);
        }

    }

    /**
     * 发生异常触发
     */
    @Override
    public void exceptionCaught(ChannelHandlerContext ctx, Throwable cause) throws Exception {
        cause.printStackTrace();
        ctx.close();
    }

    /*
     * 取得byte数组
     */
    private byte[] getReturnBytes(String astCmdBuf) {
        return HexConvert.hexStringToBytes(astCmdBuf);
    }

    private byte[] getCheckSumBytes(String astCmdBuf) {
        String checkSum = HexConvert.makeChecksum(astCmdBuf).toUpperCase();
        StringBuffer astCheckSumBuf = new StringBuffer();
        astCheckSumBuf.append(checkSum);
        while (astCheckSumBuf.length()<4) {
            astCheckSumBuf.insert(0,"0");
        }
        return getReturnBytes(astCheckSumBuf.toString());
        //log.info(checkSum);
    }


    private byte[] getReturnBytes() {
        String returnStr = "0D0A";
        return getReturnBytes(returnStr);
    }


}
